@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2003 the xine project
+ * Copyright (C) 2001-2009 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -38,8 +38,8 @@ typedef unsigned char boolean;
 #define LOG
 */
 
-#include "audio_out.h"
-#include "xine_internal.h"
+#include <xine/audio_out.h>
+#include <xine/xine_internal.h>
 
 
 #define MAX_CHANNELS	          6
@@ -52,7 +52,7 @@ typedef unsigned char boolean;
 #define DSBUFF_LEFT		  1
 #define DSBUFF_RIGHT	          2
 
-#define AO_DIRECTX_IFACE_VERSION  8
+#define AO_DIRECTX_IFACE_VERSION  9
 
 /*****************************************************************************
  * DirectDraw GUIDs.
@@ -823,20 +823,6 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
   return ( ao_driver_t * ) ao_directx;
 }
 
-static char* get_identifier (audio_driver_class_t *this_gen) {
-  return "DirectX";
-}
-
-static char* get_description (audio_driver_class_t *this_gen) {
-  return _("xine audio output plugin for win32 using directx");
-}
-
-static void dispose_class (audio_driver_class_t *this_gen) {
-  audiox_class_t  *audiox = (audiox_class_t *) this_gen;
-
-  free (audiox);
-}
-
 static void *init_class (xine_t *xine, void *data) {
   audiox_class_t    *audiox;
 
@@ -850,9 +836,9 @@ static void *init_class (xine_t *xine, void *data) {
     return NULL;
 
   audiox->driver_class.open_plugin     = open_plugin;
-  audiox->driver_class.get_identifier  = get_identifier;
-  audiox->driver_class.get_description = get_description;
-  audiox->driver_class.dispose         = dispose_class;
+  audiox->driver_class.identifier      = "DirectX";
+  audiox->driver_class.description     = N_("xine audio output plugin for win32 using directx");
+  audiox->driver_class.dispose         = default_audio_driver_class_dispose;
 
   audiox->xine                         = xine;
   audiox->config                       = xine->config;
