@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2012 the xine project
+ * Copyright (C) 2000-2014 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -315,6 +315,8 @@ struct matroska_track_s {
 #define MATROSKA_CODEC_ID_V_MJPEG        "V_MJPEG"
 #define MATROSKA_CODEC_ID_V_THEORA       "V_THEORA"
 #define MATROSKA_CODEC_ID_V_VP8          "V_VP8"
+#define MATROSKA_CODEC_ID_V_VP9          "V_VP9"
+#define MATROSKA_CODEC_ID_V_HEVC         "V_MPEGH/ISO/HEVC"
 
 #define MATROSKA_CODEC_ID_A_MPEG1_L1     "A_MPEG/L1"
 #define MATROSKA_CODEC_ID_A_MPEG1_L2     "A_MPEG/L2"
@@ -335,6 +337,7 @@ struct matroska_track_s {
 #define MATROSKA_CODEC_ID_A_REAL_RALF    "A_REAL/RALF"
 #define MATROSKA_CODEC_ID_A_REAL_ATRC    "A_REAL/ATRC"
 #define MATROSKA_CODEC_ID_A_FLAC         "A_FLAC"
+#define MATROSKA_CODEC_ID_A_OPUS         "A_OPUS"
 
 #define MATROSKA_CODEC_ID_S_TEXT_UTF8    "S_TEXT/UTF8"
 #define MATROSKA_CODEC_ID_S_TEXT_SSA     "S_TEXT/SSA"
