@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2012 the xine project
+ * Copyright (C) 2000-2014 the xine project
  * Copyright (C) 2009-2011 Petri Hintukainen <phintuka@users.sourceforge.net>
  *
  * This file is part of xine, a free video player.
@@ -317,7 +317,7 @@ static void draw_bitmap(xine_osd_t *osd, const BD_OVERLAY * const ov)
 #endif
 
   /* uncompress and draw bitmap */
-  if (ov->img) {
+  if (ov->img && ov->w > 0 && ov->h > 0) {
     const BD_PG_RLE_ELEM *rlep = ov->img;
     uint8_t *img = malloc(ov->w * ov->h);
     unsigned pixels = ov->w * ov->h;
@@ -539,9 +539,8 @@ static void update_title_name(bluray_input_plugin_t *this)
   } else if (this->current_title == BLURAY_TITLE_FIRST_PLAY) {
     strcpy(title_name, "First Play");
   } else if (this->nav_mode) {
-    snprintf(title_name, sizeof(title_name), "Title %d/%d (PL %d/%d)",
-             this->current_title, this->num_titles,
-             this->current_title_idx + 1, this->num_title_idx);
+    snprintf(title_name, sizeof(title_name), "Title %d/%d",
+             this->current_title, this->num_titles);
   } else {
     snprintf(title_name, sizeof(title_name), "Title %d/%d",
              this->current_title_idx + 1, this->num_title_idx);
@@ -582,13 +581,6 @@ static void update_title_info(bluray_input_plugin_t *this, int playlist_id)
     return;
   }
 
-#ifdef LOG
-  int ms = this->title_info->duration / INT64_C(90);
-  lprintf("Opened title %d. Length %"PRId64" bytes / %02d:%02d:%02d.%03d\n",
-          this->current_title_idx, bd_get_title_size(this->bdh),
-          ms / 3600000, (ms % 3600000 / 60000), (ms % 60000) / 1000, ms % 1000);
-#endif
-
   /* calculate and set stream rate */
 
   uint64_t rate = bd_get_title_size(this->bdh) * UINT64_C(8) // bits
@@ -823,7 +815,9 @@ static void handle_libbluray_event(bluray_input_plugin_t *this, BD_EVENT ev)
 
       case BD_EVENT_PLAYLIST:
         lprintf("BD_EVENT_PLAYLIST %d\n", ev.param);
-        this->current_title_idx = bd_get_current_title(this->bdh);
+        if (!this->nav_mode) {
+          this->current_title_idx = bd_get_current_title(this->bdh);
+        }
         this->current_clip = 0;
         update_title_info(this, ev.param);
         stream_reset(this);
@@ -912,6 +906,13 @@ static int open_title (bluray_input_plugin_t *this, int title_idx)
 
   update_title_info(this, -1);
 
+#ifdef LOG
+  int ms = this->title_info->duration / INT64_C(90);
+  lprintf("Opened title %d. Length %"PRId64" bytes / %02d:%02d:%02d.%03d\n",
+          this->current_title_idx, bd_get_title_size(this->bdh),
+          ms / 3600000, (ms % 3600000 / 60000), (ms % 60000) / 1000, ms % 1000);
+#endif
+
   return 1;
 }
 
@@ -1533,6 +1534,15 @@ static int get_disc_info(bluray_input_plugin_t *this)
                 "Some titles can't be played in navigation mode", NULL);
   }
 
+  if (this->nav_mode && disc_info->num_bdj_titles > 0) {
+    if (!(this->stream->video_out->get_capabilities(this->stream->video_out) & VO_CAP_ARGB_LAYER_OVERLAY)) {
+      _x_message (this->stream, XINE_MSG_GENERAL_WARNING,
+                  "BD-J titles found. Current video driver does not support ARGB graphics.",
+                  "Try another video driver (ex. --video opengl2) or play this disc without menus.",
+                  NULL);
+    }
+  }
+
   this->num_titles = disc_info->num_hdmv_titles + disc_info->num_bdj_titles;
   this->disc_info  = disc_info;
 
@@ -1622,28 +1632,31 @@ static int bluray_plugin_open (input_plugin_t *this_gen)
 
   /* load title list */
 
-  this->num_title_idx = bd_get_titles(this->bdh, TITLES_RELEVANT, MIN_TITLE_LENGTH);
-  LOGMSG("%d titles\n", this->num_title_idx);
+  if (!this->nav_mode) {
+    this->num_title_idx = bd_get_titles(this->bdh, TITLES_RELEVANT, MIN_TITLE_LENGTH);
+    LOGMSG("%d titles\n", this->num_title_idx);
 
-  if (this->num_title_idx < 1)
-    return -1;
+    if (this->num_title_idx < 1)
+      return -1;
 
-  /* select title */
-
-  /* if title was not in mrl, guess the main title */
-  if (title < 0) {
-    uint64_t duration = 0;
-    int i, playlist = 99999;
-    for (i = 0; i < this->num_title_idx; i++) {
-      BLURAY_TITLE_INFO *info = bd_get_title_info(this->bdh, i, 0);
-      if (info->duration > duration) {
-        title    = i;
-        duration = info->duration;
-        playlist = info->playlist;
+    /* if title was not in mrl, guess the main title */
+    if (title < 0) {
+      uint64_t duration = 0;
+      int i, playlist = 99999;
+      for (i = 0; i < this->num_title_idx; i++) {
+        BLURAY_TITLE_INFO *info = bd_get_title_info(this->bdh, i, 0);
+        if (info->duration > duration) {
+          title    = i;
+          duration = info->duration;
+          playlist = info->playlist;
+        }
+        bd_free_title_info(info);
       }
-      bd_free_title_info(info);
+      LOGMSG("main title: %d (%05d.mpls)\n", title, playlist);
     }
-    lprintf("main title: %d (%05d.mpls)\n", title, playlist);
+
+  } else {
+    LOGMSG("%d titles\n", this->num_titles);
   }
 
   /* update player settings */
