@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2011 the xine project
+ * Copyright (C) 2000-2014 the xine project
  *
  * Copyright (C) 2009 Petri Hintukainen <phintuka@users.sourceforge.net>
  *
@@ -374,10 +374,17 @@ static subtitle_clut_t *segbuf_decode_palette(segment_buffer_t *buf)
     uint8_t Cr    = segbuf_get_u8 (buf);
     uint8_t Cb    = segbuf_get_u8 (buf);
     uint8_t alpha = segbuf_get_u8 (buf);
+#ifdef WORDS_BIGENDIAN
+    clut->color[index] = (Y << 8) | (Cr << 16) | (Cb << 24);
+#else
     clut->color[index] = (Y << 16) | (Cr << 8) | Cb;
+#endif
     clut->trans[index] = alpha >> 4;
   }
 
+  /* HDMV subs always use same color matrix as main video. */
+  _X_SET_CLUT_CM (clut->color, 4);
+
   return clut;
 }
 
@@ -969,6 +976,9 @@ static void spudec_decode_data (spu_decoder_t * this_gen, buf_element_t * buf)
   if ((buf->type & 0xffff0000) != BUF_SPU_HDMV)
     return;
 
+  if ((this->stream->spu_channel & 0x1f) != (buf->type & 0x1f))
+    return;
+
   if (buf->size < 1)
     return;
 
