@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2012 the xine project
+ * Copyright (C) 2000-2014 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -155,7 +155,7 @@ typedef struct sputext_decoder_s {
   subtitle_size      subtitle_size;   /* size of subtitles */
   int                vertical_offset;
   char               font[FILENAME_MAX]; /* subtitle font */
-  char              *buf_encoding;    /* encoding of subtitle buffer */
+  const char         *buf_encoding;    /* encoding of subtitle buffer */
 
   int                width;          /* frame width                */
   int                height;         /* frame height               */
@@ -338,36 +338,43 @@ static int ogm_render_line_internal(sputext_decoder_t *this, int x, int y, const
   while (i <= length) {
 
     if (text[i] == '<') {
-      if (!strncmp("<b>", text+i, 3)) {
+      if (!strncasecmp("<b>", text+i, 3)) {
 	/* enable Bold color */
 	if (render)
 	  this->current_osd_text = OSD_TEXT2;
 	i=i+3;
 	continue;
-      } else if (!strncmp("</b>", text+i, 4)) {
+      } else if (!strncasecmp("</b>", text+i, 4)) {
 	/* disable BOLD */
 	if (render)
 	  this->current_osd_text = OSD_TEXT1;
 	i=i+4;
 	continue;
-      } else if (!strncmp("<i>", text+i, 3)) {
+      } else if (!strncasecmp("<i>", text+i, 3)) {
 	/* enable italics color */
 	if (render)
 	  this->current_osd_text = OSD_TEXT3;
 	i=i+3;
 	continue;
-      } else if (!strncmp("</i>", text+i, 4)) {
+      } else if (!strncasecmp("</i>", text+i, 4)) {
 	/* disable italics */
 	if (render)
 	  this->current_osd_text = OSD_TEXT1;
 	i=i+4;
 	continue;
-      } else if (!strncmp("<font>", text+i, 6)) {
+      } else if (!strncasecmp("<font>", text+i, 6)) {
 	/*Do somethink to disable typing
 	  fixme - no teststreams*/
 	i=i+6;
 	continue;
-      } else if (!strncmp("</font>", text+i, 7)) {
+      } else if (!strncasecmp("<font ", text+i, 6)) {
+        /* TODO: parse and use attributes. seen: <font color="#xxyyzz"> */
+        const char *end = strchr(text + i, '>');
+        if (end) {
+          i += (end - (text + i)) + 1;
+          continue;
+        }
+      } else if (!strncasecmp("</font>", text+i, 7)) {
 	/*Do somethink to enable typing
 	  fixme - no teststreams*/
 	i=i+7;
@@ -1133,7 +1140,7 @@ static void *init_spu_decoder_plugin (xine_t *xine, void *data) {
   this->subtitle_size  = xine->config->register_enum(xine->config,
 			      "subtitles.separate.subtitle_size",
 			       1,
-			       subtitle_size_strings,
+			       (char **)subtitle_size_strings,
 			       _("subtitle size"),
 			       _("You can adjust the subtitle size here. The setting will "
 			         "be evaluated relative to the window size."),
