@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2012 the xine project and Fredrik Noring
+ * Copyright (C) 2000-2014 the xine project and Fredrik Noring
  *
  * This file is part of xine, a free video player.
  *
@@ -410,39 +410,23 @@ static void fb_update_frame_format(vo_driver_t *this_gen,
 static void fb_overlay_clut_yuv2rgb(fb_driver_t *this,
 				    vo_overlay_t *overlay, fb_frame_t *frame)
 {
-  size_t i;
-  clut_t* clut = (clut_t*)overlay->color;
+  int i;
+  uint32_t *rgb;
 
-  if(!overlay->rgb_clut)
-  {
-    for(i = 0;
-	i < sizeof(overlay->color)/sizeof(overlay->color[0]);
-	i++)
-    {
-      *((uint32_t *)&clut[i]) =
-	frame->yuv2rgb->
-	yuv2rgb_single_pixel_fun(frame->yuv2rgb,
-				 clut[i].y,
-				 clut[i].cb,
-				 clut[i].cr);
+  if (!overlay->rgb_clut) {
+    rgb = overlay->color;
+    for (i = sizeof (overlay->color) / sizeof (overlay->color[0]); i > 0; i--) {
+      clut_t *yuv = (clut_t *)rgb;
+      *rgb++ = frame->yuv2rgb->yuv2rgb_single_pixel_fun (frame->yuv2rgb, yuv->y, yuv->cb, yuv->cr);
     }
     overlay->rgb_clut++;
   }
 
-  if(!overlay->hili_rgb_clut)
-  {
-    clut = (clut_t*) overlay->hili_color;
-
-    for(i = 0;
-	i < sizeof(overlay->color)/sizeof(overlay->color[0]);
-	i++)
-    {
-      *((uint32_t *)&clut[i]) =
-	frame->yuv2rgb->
-	yuv2rgb_single_pixel_fun(frame->yuv2rgb,
-				 clut[i].y,
-				 clut[i].cb,
-				 clut[i].cr);
+  if (!overlay->hili_rgb_clut) {
+    rgb = overlay->hili_color;
+    for (i = sizeof (overlay->color) / sizeof (overlay->color[0]); i > 0; i--) {
+      clut_t *yuv = (clut_t *)rgb;
+      *rgb++ = frame->yuv2rgb->yuv2rgb_single_pixel_fun (frame->yuv2rgb, yuv->y, yuv->cb, yuv->cr);
     }
     overlay->hili_rgb_clut++;
   }
@@ -989,12 +973,21 @@ static vo_driver_t *fb_open_plugin(video_driver_class_t *class_gen,
   this->fd = open_fb_device(config, class->xine);
   if(this->fd == -1)
     goto error;
-  if(!get_fb_var_screeninfo(this->fd, &this->fb_var, class->xine))
+  if(!get_fb_var_screeninfo(this->fd, &this->fb_var, class->xine)) {
+    xprintf(this->xine, XINE_VERBOSITY_LOG, "video_out_fb: get_fb_var_screeninfo() failed: %s\n", strerror(errno));
     goto error;
-  if(!get_fb_fix_screeninfo(this->fd, &this->fb_fix, class->xine))
-    goto error;
-  if (!set_fb_palette (this->fd, &this->fb_var))
+  }
+  if(!get_fb_fix_screeninfo(this->fd, &this->fb_fix, class->xine)) {
+    xprintf(this->xine, XINE_VERBOSITY_LOG, "video_out_fb: get_fb_fix_screeninfo() failed: %s\n", strerror(errno));
     goto error;
+  }
+
+  if (!set_fb_palette (this->fd, &this->fb_var)) {
+    xprintf(this->xine, XINE_VERBOSITY_LOG, "video_out_fb: set_fb_palette() failed: %s\n", strerror(errno));
+    if (this->fb_var.bits_per_pixel < 24) {
+      goto error;
+    }
+  }
 
   this->xine = class->xine;
 
@@ -1040,6 +1033,9 @@ static vo_driver_t *fb_open_plugin(video_driver_class_t *class_gen,
 			      MAP_SHARED, this->fd, 0);
   return &this->vo_driver;
 error:
+  xprintf(class->xine, XINE_VERBOSITY_DEBUG,
+          "video_out_fb: Unable to configure fb device, aborting: %s\n", strerror(errno));
+
   free(this);
   return 0;
 }
