@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2009 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -41,10 +41,10 @@
 #include <X11/Xatom.h>
 #include <dga/dga.h>
 
-#include "xine_internal.h"
+#include <xine/xine_internal.h>
 #include "bswap.h"
-#include "vo_scale.h"
-#include "xineutils.h"
+#include <xine/vo_scale.h>
+#include <xine/xineutils.h>
 
 /* gfxp register defines */
 
@@ -783,13 +783,6 @@ static void pgx32_dispose(vo_driver_t *this_gen)
  * XINE VIDEO DRIVER CLASS FUNCTIONS
  */
 
-static void pgx32_dispose_class(video_driver_class_t *class_gen)
-{
-  pgx32_driver_class_t *class = (pgx32_driver_class_t *)(void *)class_gen;
-
-  free(class);
-}
-
 static const vo_info_t vo_info_pgx32 = {
   10,
   XINE_VISUAL_TYPE_X11
@@ -855,16 +848,6 @@ static vo_driver_t *pgx32_init_driver(video_driver_class_t *class_gen, const voi
   return (vo_driver_t *)this;
 }
 
-static char *pgx32_get_identifier(video_driver_class_t *class_gen)
-{
-  return "pgx32";
-}
-
-static char *pgx32_get_description(video_driver_class_t *class_gen)
-{
-  return "xine video output plugin for Sun PGX32 framebuffers";
-}
-
 static void *pgx32_init_class(xine_t *xine, void *visual_gen)
 {
   pgx32_driver_class_t *class;
@@ -877,9 +860,9 @@ static void *pgx32_init_class(xine_t *xine, void *visual_gen)
   DGA_INIT();
 
   class->vo_driver_class.open_plugin     = pgx32_init_driver;
-  class->vo_driver_class.get_identifier  = pgx32_get_identifier;
-  class->vo_driver_class.get_description = pgx32_get_description;
-  class->vo_driver_class.dispose         = pgx32_dispose_class;
+  class->vo_driver_class.identifier      = "pgx32";
+  class->vo_driver_class.description     = N_("xine video output plugin for Sun PGX32 framebuffers");
+  class->vo_driver_class.dispose         = default_video_driver_class_dispose;
 
   class->xine   = xine;
   class->config = xine->config;
@@ -888,6 +871,6 @@ static void *pgx32_init_class(xine_t *xine, void *visual_gen)
 }
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
-  {PLUGIN_VIDEO_OUT, 21, "pgx32", XINE_VERSION_CODE, &vo_info_pgx32, pgx32_init_class},
+  {PLUGIN_VIDEO_OUT, 22, "pgx32", XINE_VERSION_CODE, &vo_info_pgx32, pgx32_init_class},
   {PLUGIN_NONE, 0, "", 0, NULL, NULL}
 };
