@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2012 the xine project
+ * Copyright (C) 2001-2014 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -508,6 +508,12 @@ static uint64_t memcpy_timing[sizeof(memcpy_method)/sizeof(memcpy_method[0])] =
 
 #ifdef HAVE_POSIX_TIMERS
 /* Prefer clock_gettime() where available. */
+
+# ifndef CLOCK_THREAD_CPUTIME_ID
+/*  not defined in NetBSD (bug #535) */
+#   define CLOCK_THREAD_CPUTIME_ID CLOCK_MONOTONIC
+# endif
+
 static int64_t _x_gettime(void)
 {
   struct timespec tm;
@@ -589,7 +595,7 @@ void xine_probe_fast_memcpy(xine_t *xine)
   config_flags = xine_mm_accel();
 
   best = xine->config->register_enum (xine->config, "engine.performance.memcpy_method", 0,
-				      memcpy_methods,
+				      (char **)memcpy_methods,
 				      _("memcopy method used by xine"),
 				      _("The copying of large memory blocks is one of the most "
 					"expensive operations on todays computers. Therefore xine "
